import sys
import traceback
import uuid

class baseException(Exception):
    
    """
    例外クラス(基底)
    """
    
    def __init__(self):
        
        """
        コンストラクタ
        """
        
        super().__init__()
        
        # 例外情報取得
        exc_type, exc_value, exc_traceback = sys.exc_info()
        self.__exc_type = exc_type
        self.__exc_value = exc_value
        self.__exc_traceback = ''.join(traceback.format_exception(exc_type, exc_value, exc_traceback))
        self.__exc_uuid = str(uuid.uuid4())
    
    @property
    def exc_type(self):
        return self.__exc_type
    
    @property
    def exc_value(self):
        return self.__exc_value
    
    @property
    def exc_traceback(self):
        return self.__exc_traceback

    @property
    def exc_uuid(self):
        return self.__exc_uuid

class getterError(baseException):
    
    """
    getter例外クラス
    """
    
    def __init__(self):
        
        """
        コンストラクタ
        """
        
        super().__init__()
    
    def __str__(self):
        
        """
        例外をプリントした時に出力する文字列
        """

        return "getter runtime error"

class registerError(baseException):
    
    """
    register例外クラス
    """
    
    def __init__(self):
        
        """
        コンストラクタ
        """
        
        super().__init__()
    
    def __str__(self):
        
        """
        例外をプリントした時に出力する文字列
        """

        return "register runtime error"

class resultDataGenerateError(baseException):

    """
    検索結果生成時例外クラス
    """

    def __init__(self):

        """
        コンストラクタ
        """

        super().__init__()

    def __str__(self):
        
        """
        例外をプリントした時に出力する文字列
        """

        return "Failed to generate search dictionary"
    
class arrivalOrderValueError(getterError):
    
    """
    着順構造体値エラー時例外クラス
    """
    
    def __init__(self):

        """
        コンストラクタ
        """

        super().__init__()

    def __str__(self):
        
        """
        例外をプリントした時に出力する文字列
        """

        return "Arraival Order Value Error"
    
class racdIdCheckError(getterError):
    
    """
    レースIDチェックエラー例外クラス
    """
    
    def __init__(self):

        """
        コンストラクタ
        """

        super().__init__()

    def __str__(self):
        
        """
        例外をプリントした時に出力する文字列
        """

        return "race_id is not digit"

class gettingValueError(getterError):
    
    """
    取得値エラー例外クラス
    """
    
    def __init__(self):

        """
        コンストラクタ
        """

        super().__init__()

    def __str__(self):
        
        """
        例外をプリントした時に出力する文字列
        """

        return "value is invaild"
    



from Scrapinger.Library.scrapingerGlobals import scrapingerGlobal
from netkeiber.Library.netkeiberConfig import netkeiberConfig

class netkeiberGlobal(scrapingerGlobal):
    
    def __init__(self):
        
        """
        コンストラクタ
        """
        
        # 基底側コンストラクタ呼び出し
        super().__init__()

        self.netkeiberConfig:netkeiberConfig = None
        """ netkeiber共通設定 """

# インスタンス生成(import時に実行される)
gv = netkeiberGlobal()



import LibHanger.Models.modelFields as fld
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.sql.elements import Null

# Baseクラス生成
Base = declarative_base()

class trn_open_info(Base):
	
	# テーブル名
	__tablename__ = 'trn_open_info'
	
	# スキーマ
	__table_args__ = {'schema': 'nkaz'}
	
	# 列定義
	open_id = fld.CharFields(8,primary_key=True,default='')
	racecourse_id = fld.CharFields(2,primary_key=True,default='')
	race_no = fld.NumericFields(2,0,primary_key=True,default=0)
	race_id = fld.CharFields(12,default='')
	win_horse = fld.CharFields(10,default='')
	win_jockey = fld.CharFields(5,default='')
	snd_horse = fld.CharFields(10,default='')
	snd_jockey = fld.CharFields(5,default='')
	trd_horse = fld.CharFields(10,default='')
	trd_jockey = fld.CharFields(5,default='')
	updinfo = fld.CharFields(40,default='')



import LibHanger.Library.uwLogger as Logger
import LibHanger.Library.uwGetter as CmnGetter
from enum import Enum
from LibHanger.Models.recset import recset
from netkeiber.Getter.Base.baseGetter import baseGetter
from netkeiber.Models.log_race_id import log_race_id
from netkeiber.Library.netkeiberException import baseException

class getter_log_race_id(baseGetter):
    
    """
    レースIDログクラス
    (log_race_id)
    """
    
    def __init__(self) -> None:
        
        """
        コンストラクタ
        """
        
        # 基底側コンストラクタ
        super().__init__()

        # レコードセット初期化
        self.init_recset()
    
    def init_recset(self):
        
        """
        レコードセット初期化
        """

        # レコードセット初期化
        self.rsLogRaceId = recset[log_race_id](log_race_id)
        
    def setLogRaceId(self, race_id, getterResult:Enum, e:baseException = None):
        
        """
        レースIDログ-レコードセット生成
        
        Parameters
        ----------
        race_id : str
            レースID
        getterResult : netkeiberDeclare.getterResult
            処理結果
        e : getterError
            getter例外
        """
        
        # レースIDログ生成
        self.rsLogRaceId.newRow()
        self.rsLogRaceId.fields(log_race_id.race_id.key).value = race_id
        self.rsLogRaceId.fields(log_race_id.create_datetime.key).value = CmnGetter.getNow()
        self.rsLogRaceId.fields(log_race_id.result.key).value = getterResult.value
        if not e is None and isinstance(e,baseException):
            self.rsLogRaceId.fields(log_race_id.uuid.key).value = e.exc_uuid
            self.rsLogRaceId.fields(log_race_id.type.key).value = str(e.exc_type)[:50]
            self.rsLogRaceId.fields(log_race_id.value.key).value = str(e.exc_value)[:50]
            self.rsLogRaceId.fields(log_race_id.stacktrace.key).value = e.exc_traceback[:2000]
        elif not e is None:
            self.rsLogRaceId.fields(log_race_id.stacktrace.key).value = str(e)
        self.rsLogRaceId.fields(log_race_id.updinfo.key).value = CmnGetter.getNow(CmnGetter.datetimeFormat.updinfo)
        
        # ログ出力
        if not e is None and isinstance(e,baseException):
            Logger.logging.error(e.exc_uuid)
            Logger.logging.error(e.exc_type)
            Logger.logging.error(e.exc_value)
            Logger.logging.error(e.exc_traceback)
        elif not e is None:
            Logger.logging.error(str(e))
            


import LibHanger.Models.modelFields as fld
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.sql.elements import Null

# Baseクラス生成
Base = declarative_base()

class trn_horse_result(Base):
	
	# テーブル名
	__tablename__ = 'trn_horse_result'
	
	# スキーマ
	__table_args__ = {'schema': 'nkaz'}
	
	# 列定義
	horse_id = fld.CharFields(10,primary_key=True,default='')
	race_id = fld.CharFields(12,primary_key=True,default='')
	horse_no = fld.NumericFields(2,0,primary_key=True,default=0)
	racecourse_id = fld.CharFields(2,default='')
	open_id = fld.CharFields(8,default='')
	open_nm = fld.CharFields(16,default='')
	pace_firsth = fld.NumericFields(3,1,default=0)
	pace_latterh = fld.NumericFields(3,1,default=0)
	win_horse_id = fld.CharFields(12,default='')
	updinfo = fld.CharFields(40,default='')



import LibHanger.Models.modelFields as fld
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.sql.elements import Null

# Baseクラス生成
Base = declarative_base()

class trn_refund_info(Base):
	
	# テーブル名
	__tablename__ = 'trn_refund_info'
	
	# スキーマ
	__table_args__ = {'schema': 'nkaz'}
	
	# 列定義
	racecourse_id = fld.CharFields(2,primary_key=True,default='')
	refund_id = fld.CharFields(10,primary_key=True,default='')
	race_no = fld.NumericFields(2,0,primary_key=True,default=0)
	refund_kbn = fld.NumericFields(1,0,primary_key=True,default=0)
	refund_seq = fld.NumericFields(1,0,primary_key=True,default=0)
	horse_no = fld.CharFields(20,default='')
	pay = fld.NumericFields(9,0,default=0)
	pop = fld.NumericFields(4,0,default=0)
	updinfo = fld.CharFields(40,default='')



import LibHanger.Models.modelFields as fld
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.sql.elements import Null

# Baseクラス生成
Base = declarative_base()

class trn_race_info(Base):
	
	# テーブル名
	__tablename__ = 'trn_race_info'
	
	# スキーマ
	__table_args__ = {'schema': 'nkaz'}
	
	# 列定義
	race_id = fld.CharFields(12,primary_key=True,default='')
	race_no = fld.NumericFields(2,0,default=0)
	race_nm = fld.CharFields(100,default='')
	grade = fld.NumericFields(1,0,default=0)
	course_summary = fld.CharFields(30,default='')
	direction = fld.CharFields(4,default='')
	distance = fld.NumericFields(4,0,default=0)
	ground_kbn = fld.NumericFields(1,0,default=0)
	weather = fld.CharFields(3,default='')
	ground_cond = fld.CharFields(3,default='')
	ground_cond_d = fld.CharFields(3,default='')
	race_time = fld.CharFields(5,default='')
	race_date = fld.DateFields(default=Null)
	open_info = fld.CharFields(30,default='')
	class_nm = fld.CharFields(50,default='')
	horse_sign = fld.CharFields(30,default='')
	head_count = fld.NumericFields(2,0,default=0)
	racecourse_id = fld.CharFields(2,default='')
	refund_id = fld.CharFields(10,default='')
	updinfo = fld.CharFields(40,default='')



import LibHanger.Models.modelFields as fld
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.sql.elements import Null

# Baseクラス生成
Base = declarative_base()

class log_race_id(Base):
	
	# テーブル名
	__tablename__ = 'log_race_id'
	
	# スキーマ
	__table_args__ = {'schema': 'nkaz'}
	
	# 列定義
	race_id = fld.CharFields(12,primary_key=True,default='')
	create_datetime = fld.DateTimeFields(default=Null)
	result = fld.NumericFields(1,0,default=0)
	uuid = fld.CharFields(40,default='')
	type = fld.CharFields(50,default='')
	value = fld.CharFields(50,default='')
	stacktrace = fld.CharFields(2000,default='')
	error_point = fld.CharFields(30,default='')
	updinfo = fld.CharFields(40,default='')



import LibHanger.Models.modelFields as fld
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.sql.elements import Null

# Baseクラス生成
Base = declarative_base()

class trn_race_id(Base):
	
	# テーブル名
	__tablename__ = 'trn_race_id'
	
	# スキーマ
	__table_args__ = {'schema': 'nkaz'}
	
	# 列定義
	race_id = fld.CharFields(12,primary_key=True,default='')
	racecourse_id = fld.CharFields(2,primary_key=True,default='')
	open_id = fld.CharFields(8,primary_key=True,default='')
	scraping_count = fld.NumericFields(3,0,default=0)
	get_time = fld.NumericFields(4,0,default=0)
	get_status = fld.NumericFields(1,0,default=0)
	updinfo = fld.CharFields(40,default='')



import LibHanger.Models.modelFields as fld
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.sql.elements import Null

# Baseクラス生成
Base = declarative_base()

class mst_horse_sign(Base):
	
	# テーブル名
	__tablename__ = 'mst_horse_sign'
	
	# スキーマ
	__table_args__ = {'schema': 'nkaz'}
	
	# 列定義
	horse_sign_id = fld.CharFields(2,primary_key=True,default='')
	horse_sign_nm = fld.CharFields(20,default='')
	updinfo = fld.CharFields(40,default='')



import LibHanger.Library.uwLogger as Logger
from sqlalchemy.exc import SQLAlchemyError
from LibHanger.Library.uwGlobals import *
from LibHanger.Library.DataAccess.uwPostgres import uwPostgreSQL
from LibHanger.Library.uwDeclare import uwDeclare as en
from LibHanger.Models.recset import recset
from netkeiber.Library.netkeiberDeclare import netkeiberDeclare as nd
from netkeiber.Library.netkeiberException import registerError
from netkeiber.Getter.log_race_id import getter_log_race_id

class baseRegister():
    
    """
    登録処理基底クラス
    """
    
    def __init__(self, __psgr:uwPostgreSQL, __race_id):
        
        """
        コンストラクタ
        
        Parameters
        ----------
        __psgr : uwPostgreSQL
            uwPostgreSQLクラスインスタンス
        __race_id : str
            レースID
        """
        
        # uwPostgreSQL
        self.__psgr = __psgr
        
        # recset
        self.__recsetList = []
    
        # レースIDログ
        self.raceIdLog = getter_log_race_id()
        
        # レースID
        self.race_id = __race_id
        
        # エラー有無
        self.__hasError = False

    @property
    def hasError(self):
        
        """
        エラー有無
        """
        
        return self.__hasError

    def appendRecsetList(self, __recset:recset):
        
        """
        登録対象レコードセットリストの追加

        Parameters
        ----------
        __recset : recset
            レコードセットクラスインスタンス

        """
        
        # 登録対象リストに追加
        self.__recsetList.append(__recset)
    
    def execSqlfile(self, sql):

        """
        SQLファイルを実行する

        Parameters
        ----------
        sql : str
            実行するSQL

        """
        
        # 結果セット
        result = en.resultRegister.success
        
        try:
            # sql - execute
            self.__psgr.sqlExecute(sql)

            # Commit
            self.__psgr.commit()

        except SQLAlchemyError as e:
            
            # 結果セット
            result = en.resultRegister.failure
            # ログ出力
            Logger.logging.error(e)
            # Rollback
            self.__psgr.rollback()
        
        # 処理結果を返す
        return result
    
    def execUpdate(self):
        
        """
        DB更新処理
        """
        
        # open - session
        self.__psgr.openSession()

        # begin - Transaction
        self.__psgr.beginTransaction()
        
        # レコードセットupdate
        for rs in self.__recsetList:
            rsx:recset = rs
            rsx.setDA(self.__psgr)
            upResult = rsx.upsert()
            if upResult.result == en.resultRegister.failure : break

        # 結果判定
        if upResult.result == en.resultRegister.success:
            # commit
            self.__psgr.commit()
            # レースIDログ追加
            self.raceIdLog.setLogRaceId(self.race_id, nd.getterResult.success)

        elif upResult.result == en.resultRegister.failure:
            try:
                try:
                    # rollback
                    self.__psgr.rollback()
                    # raise
                    raise upResult.exceptInfo
                except SQLAlchemyError as e:
                    raise registerError
            except registerError as e:
                # レースIDログ追加
                self.raceIdLog.setLogRaceId(self.race_id, nd.getterResult.error, e)
        
        # session - close
        self.__psgr.closeSession()
        
        # エラー有無セット
        self.__hasError = False if upResult.result == en.resultRegister.success else True
        
        # 結果を返す
        return upResult.result
        


import LibHanger.Library.uwLogger as Logger
from LibHanger.Library.DataAccess.uwPostgres import uwPostgreSQL
from netkeiber.Register.Base.baseRegister import baseRegister
from LibHanger.Models.recset import recset

class register_trn_race_result(baseRegister):

    def __init__(self, __psgr: uwPostgreSQL, __recset: recset):
        super().__init__(__psgr, __recset)
        
        self.recset = __recset


import LibHanger.Models.modelFields as fld
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.sql.elements import Null

# Baseクラス生成
Base = declarative_base()

class mst_horse(Base):
	
	# テーブル名
	__tablename__ = 'mst_horse'
	
	# スキーマ
	__table_args__ = {'schema': 'nkaz'}
	
	# 列定義
	horse_id = fld.CharFields(10,primary_key=True,default='')
	horse_nm = fld.CharFields(30,primary_key=True,default='')
	horse_nm_en = fld.CharFields(40,default='')
	horse_sign_id = fld.CharFields(2,default='')
	birthday = fld.DateFields(default=Null)
	sex = fld.CharFields(6,default='')
	trainer_id = fld.CharFields(6,default='')
	howner_id = fld.CharFields(7,default='')
	breeder_id = fld.CharFields(7,default='')
	producing_area = fld.CharFields(30,default='')
	trn_price = fld.NumericFields(9,0,default=0)
	trn_note = fld.CharFields(30,default='')
	peds_id = fld.CharFields(12,default='')
	updinfo = fld.CharFields(40,default='')



import urllib.parse as urlParse
import LibHanger.Library.uwLogger as Logger
from pandas.core.frame import DataFrame
from bs4 import BeautifulSoup, ResultSet, Tag
from LibHanger.Models.recset import recset
from Scrapinger.Library.browserContainer import browserContainer
from netkeiber.Library.netkeiberConfig import netkeiberConfig
from netkeiber.Library.netkeiberDeclare import netkeiberDeclare as nd
from netkeiber.Library.netkeiberGlobals import *
from netkeiber.Library.netkeiberException import getterError
from netkeiber.Library.netkeiberException import gettingValueError
from netkeiber.Models.trn_refund_info import trn_refund_info
from netkeiber.Getter.Base.baseGetter import baseGetter

class getter_trn_refund_info(baseGetter):
    
    """
    払戻情報取得クラス
    (trn_refund_info)
    """
    
    class payInfo():
        
        """
        払戻情報
        """
        
        seq:int = 0
        """ 連番 """

        horse_no:str = ''
        """ 馬番 """

        pay:int = 0
        """ 払戻金 """
        
        pop:int = 0
        """ 人気 """
        
    def __init__(self) -> None:
        
        """
        コンストラクタ
        """
        
        super().__init__()

        # レコードセット初期化
        self.init_recset()
    
    def init_recset(self):
        
        """
        レコードセット初期化
        """

        # レコードセット初期化
        self.rsRefundInfo = recset[trn_refund_info](trn_refund_info)

    @Logger.loggerDecorator("getData",['refund_id','racecourse_id'])
    def getData(self, *args, **kwargs):
        
        """
        払戻情報取得
        
        Parameters
        ----------
        kwargs : dict
            @refund_id
                払戻ID
        """
        
        # 払戻情報をDataFrameで取得
        try:
            kwargs['getter'] = self
            result = self.getRefundInfoDataToDataFrame(**kwargs)
        except:
            raise getterError
        return result
    
    @Logger.loggerDecorator("getRefundInfoDataToDataFrame")
    def getRefundInfoDataToDataFrame(self, *args, **kwargs):

        """
        払戻情報取得
        
        Parameters
        ----------
        kwargs : dict
            @refund_id
                払戻ID
        """
        
        # 検索url(ルート)
        rootUrl = urlParse.urljoin(gv.netkeiberConfig.netkeibaUrl, gv.netkeiberConfig.netkeibaUrlSearchKeyword.refund)
        # 検索url(払戻情報)
        refundInfoUrl = urlParse.urljoin(rootUrl, kwargs.get('racecourse_id') + '/' + kwargs.get('refund_id'))

        # スクレイピング準備
        self.wdc.settingScrape()

        # ページロード
        self.wdc.browserCtl.loadPage(refundInfoUrl)

        # pandasデータを返却する
        return self.wdc.browserCtl.createSearchResultDataFrame(**kwargs)
    
    def getHorseNo(self, horseNoString:str) -> str:
        
        """
        馬番取得
        
        Parameters
        ----------
        horseNoString : str
            馬番(文字列)
        """
        
        return horseNoString.replace(' ','')
    
    def getPay(self, payString:str) -> int:
        
        """
        払戻金取得

        Parameters
        ----------
        payString : str
            払戻金(文字列)
        """
        
        pay = 0
        
        try:
            pay = int(payString.replace(',','')) if payString != '' else 0
        except:
            raise gettingValueError
    
        return pay
    
    def getPop(self, popString:str) -> int:
        
        """
        人気取得
        
        Parameters
        ----------
        popString : str
            人気(文字列)
        """
        
        pop = 0
        
        try:
            pop = int(popString.replace(' ','')) if popString != '' else 0
        except:
            raise gettingValueError

        return pop
    
    def addRecsetPayInfo(self, 
                         refund_id,
                         racecourse_id,
                         race_no,
                         refund_kbn,
                         piList:list[payInfo], 
                         refundInfo:recset):
        
        """
        払戻情報をレコードセットに追加

        Parameters
        ----------
        refund_id : str
            払戻ID
        racecourse_id : str
            競馬場ID
        race_no : int
            レース番号
        refund_kbn : str
            払戻区分
        piList : list[payInfo]
            払戻情報リスト
        refundInfo : recset
            払戻情報レコードセット

        """
        
        for pi in piList:
            refundInfo.newRow()
            refundInfo.fields(trn_refund_info.refund_id.key).value = refund_id
            refundInfo.fields(trn_refund_info.racecourse_id.key).value = racecourse_id
            refundInfo.fields(trn_refund_info.race_no.key).value = race_no
            refundInfo.fields(trn_refund_info.refund_kbn.key).value = refund_kbn
            refundInfo.fields(trn_refund_info.refund_seq.key).value = pi.seq
            refundInfo.fields(trn_refund_info.horse_no.key).value = pi.horse_no
            refundInfo.fields(trn_refund_info.pay.key).value = pi.pay
            refundInfo.fields(trn_refund_info.pop.key).value = pi.pop
            refundInfo.fields(trn_refund_info.updinfo.key).value = self.getUpdInfo()
    
    def getPayInfoSingle(self, payInfoTable):
        
        """
        払戻情報取得(単)
        """
        
        # 払戻情報初期化
        payInfoSingle = [self.payInfo]
        payInfoSingle.clear()
        
        # 払戻テーブルのループ回数取得
        loopCount = len(payInfoTable[0].contents)
        
        # 連番初期化
        seq = 0
        for contents_idx in range(0,loopCount,1):
            
            if payInfoTable[0].contents[contents_idx].name == 'br': continue
            
            pi = self.payInfo()

            # 連番
            seq += 1
            pi.seq = seq
            # 馬番
            pi.horse_no = self.getHorseNo(str(payInfoTable[0].contents[contents_idx]))
            # 払戻
            pi.pay = self.getPay(str(payInfoTable[1].contents[contents_idx]))
            # 人気
            pi.pop = self.getPop(str(payInfoTable[2].contents[contents_idx]))
            
            # 払戻情報をリストに追加
            payInfoSingle.append(pi)
            
        # 戻り値を返す
        return payInfoSingle
    
    def getPayInfoDouble(self, payInfoTable):
        
        """
        払戻情報取得(複)
        """
        
        # 1組目
        horse_no1:str = self.getHorseNo(str(payInfoTable[0].contents[0]))
        pay1:int = self.getPay(str(payInfoTable[1].contents[0]))
        pop1:int = self.getPop(str(payInfoTable[2].contents[0]))

        # 2組目
        horse_no2:str = self.getHorseNo(str(payInfoTable[0].contents[2]))
        pay2:int = self.getPay(str(payInfoTable[1].contents[2]))
        pop2:int = self.getPop(str(payInfoTable[2].contents[2]))

        # 3組目
        horse_no3:str = self.getHorseNo(str(payInfoTable[0].contents[4]) if len(payInfoTable[0].contents) > 4 else '')
        pay3:int = self.getPay(str(payInfoTable[1].contents[4]) if len(payInfoTable[1].contents) > 4 else '')
        pop3:int = self.getPop(str(payInfoTable[2].contents[4]) if len(payInfoTable[2].contents) > 4 else '')
        
        # 戻り値を返す
        return horse_no1, pay1, pop1, horse_no2, pay2, pop2, horse_no3, pay3, pop3
    
    class beautifulSoup(browserContainer.beautifulSoup):
        
        """
        ブラウザコンテナ:beautifulSoup
        """

        def __init__(self, _config: netkeiberConfig):
            
            """
            コンストラクタ
            
            Parameters
            ----------
                _config : netkeiberConfig
                    共通設定
            """

            super().__init__(_config)
            
            self.config = _config
            self.cbCreateSearchResultDataFrameByBeutifulSoup = self.createSearchResultDataFrameByBeutifulSoup
            
        def createSearchResultDataFrameByBeutifulSoup(self, soup:BeautifulSoup, *args, **kwargs) -> DataFrame:
            
            """
            払戻情報をDataFrameで返す(By BeutifulSoup)
            
            Parameters
            ----------
            soup : BeautifulSoup
                BeautifulSoupオブジェクト
            
            kwargs : dict
                @refund_id
                    取得対象払戻ID
            """

            # 払戻テーブル項目名
            pil = gv.netkeiberConfig.settingValueStruct.PayInfoClass
            
            # 払戻ID取得
            refund_id:str = kwargs.get('refund_id')
            # 競馬場ID取得
            racecourse_id:str = kwargs.get('racecourse_id')

            # getterインスタンス取得
            self.bc:getter_trn_refund_info = kwargs.get('getter')
            
            # スクレイピング結果から改行ｺｰﾄﾞを除去
            [tag.extract() for tag in soup(string='\n')]
            
            # class取得
            tables = soup.find_all(class_="race_payback_info")

            if tables:
                
                # 払戻情報model用意
                refundInfo = recset[trn_refund_info](trn_refund_info)
                
                # WebBrowzerController取得
                bc = self.bc
                
                for tables_idx in range(len(tables)):
                    
                    try:
                        
                        # 払戻情報テーブルクラス取得
                        refundInfoTbl:Tag = tables[tables_idx]

                        # dtタグ検索
                        dt = refundInfoTbl.find('dt')
                        # レース番号取得
                        race_no = 0
                        if len(dt.contents) > 0:
                            race_no = int(dt.contents[0].text.replace('R',''))
                        else:
                            raise gettingValueError
                            
                        # 払戻情報テーブルクラス取得
                        payInfo = refundInfoTbl.find_all(class_="pay_table_01")
                        
                        # 払戻情報テーブルには[pay_table_01]クラスが2つ存在する(1つの場合もある)ので
                        # ループしてそれぞれの払戻種類を取得する
                        for payInfo_idx in range(len(payInfo)):
                            
                            # trタグ取得
                            payInfoTbl:Tag = payInfo[payInfo_idx]
                            payInfoTables = payInfoTbl.find_all('tr')
                            for tableIndex in range(len(payInfoTables)):
                                
                                # 払戻情報テーブル取得
                                payInfoRow:Tag = payInfoTables[tableIndex]
                                payInfoTable = payInfoRow.find_all('td')
                                
                                # 単勝
                                tan = payInfoRow.find(class_=pil.win)
                                if tan:
                                    
                                    # 馬番,払戻,人気
                                    piList = bc.getPayInfoSingle(payInfoTable)
                                    
                                    # レコードセットに追加
                                    bc.addRecsetPayInfo(refund_id, 
                                                        racecourse_id, 
                                                        race_no, 
                                                        nd.refundKbn.win.value, 
                                                        piList, 
                                                        refundInfo)
                                    
                                # 複勝
                                fuku = payInfoRow.find(class_=pil.dwin)
                                if fuku:
                                    
                                    # 馬番,払戻,人気
                                    piList = bc.getPayInfoSingle(payInfoTable)
                                    
                                    # レコードセットに追加
                                    bc.addRecsetPayInfo(refund_id, 
                                                        racecourse_id, 
                                                        race_no, 
                                                        nd.refundKbn.dwin.value, 
                                                        piList, 
                                                        refundInfo)
                                # 枠連
                                waku = payInfoRow.find(class_=pil.fcwin)
                                if waku:
                                    
                                    # 馬番,払戻,人気
                                    piList = bc.getPayInfoSingle(payInfoTable)
                                    
                                    # レコードセットに追加
                                    bc.addRecsetPayInfo(refund_id, 
                                                        racecourse_id, 
                                                        race_no, 
                                                        nd.refundKbn.fcwin.value, 
                                                        piList, 
                                                        refundInfo)
                                
                                # 馬連
                                uren = payInfoRow.find(class_=pil.hcwin)
                                if uren:
                                    
                                    # 馬番,払戻,人気
                                    piList = bc.getPayInfoSingle(payInfoTable)
                                    
                                    # レコードセットに追加
                                    bc.addRecsetPayInfo(refund_id, 
                                                        racecourse_id, 
                                                        race_no,
                                                        nd.refundKbn.hcwin.value, 
                                                        piList, 
                                                        refundInfo)
                                    
                                # ワイド
                                wide = payInfoRow.find(class_=pil.wdwin)
                                if wide:
                                    
                                    # 馬番,払戻,人気
                                    piList = bc.getPayInfoSingle(payInfoTable)
                                    
                                    # レコードセットに追加
                                    bc.addRecsetPayInfo(refund_id, 
                                                        racecourse_id, 
                                                        race_no, 
                                                        nd.refundKbn.wdwin.value, 
                                                        piList, 
                                                        refundInfo)
                                
                                # 馬単
                                utan = payInfoRow.find(class_=pil.hswin)
                                if utan:
                                    
                                    # 馬番,払戻,人気
                                    piList = bc.getPayInfoSingle(payInfoTable)
                                    
                                    # レコードセットに追加
                                    bc.addRecsetPayInfo(refund_id, 
                                                        racecourse_id, 
                                                        race_no, 
                                                        nd.refundKbn.hswin.value, 
                                                        piList, 
                                                        refundInfo)
                                
                                # 三連複
                                sanfuku = payInfoRow.find(class_=pil.tdwin)
                                if sanfuku:
                                    
                                    # 馬番,払戻,人気
                                    piList = bc.getPayInfoSingle(payInfoTable)
                                    
                                    # レコードセットに追加
                                    bc.addRecsetPayInfo(refund_id, 
                                                        racecourse_id, 
                                                        race_no, 
                                                        nd.refundKbn.tdwin.value, 
                                                        piList, 
                                                        refundInfo)
                                
                                # 三連単
                                santan = payInfoRow.find(class_=pil.tswin)
                                if santan:
                                    
                                    # 馬番,払戻,人気
                                    piList = bc.getPayInfoSingle(payInfoTable)

                                    # レコードセットに追加
                                    bc.addRecsetPayInfo(refund_id, 
                                                        racecourse_id, 
                                                        race_no, 
                                                        nd.refundKbn.tswin.value, 
                                                        piList, 
                                                        refundInfo)

                    except gettingValueError as gvException: # 値例外
                        Logger.logging.error(str(gvException))
                        raise 
                    except Exception as e: # その他例外
                        Logger.logging.error(str(e))
                        raise 

            # コンソール出力
            print('払戻ID={0}'.format(refund_id))
            
            # レコードセットマージ
            self.bc.rsRefundInfo.merge(refundInfo)

            # 戻り値を返す
            return refundInfo.getDataFrame()
            